#!/usr/bin/python
# -*- coding: UTF-8 -*-
from subprocess import Popen, PIPE, STDOUT
from re import search, DOTALL
from xml.dom.minidom import parseString

from onesheet.ClassFileMetadata import FileMetadata
from abc import ABCMeta
from onesheet.OExceptions import NoDataException


class TimeBasedMetadata(FileMetadata):
    __metaclass__ = ABCMeta
    def __init__(self, filename):
        FileMetadata.__init__(self, filename)
        self.raw_stderr = ""
        GET_XML_COMMAND = 'ffprobe -print_format xml -show_streams'.split()
        self.fileName = filename
        if self.format_type == 'audio' or self.format_type == 'video':
            command = GET_XML_COMMAND
            command.append(self.fileName)
            try:
                p = Popen(command, stdout=PIPE, stderr=PIPE, bufsize=0, universal_newlines=True)
            except OSError:
                raise IOError("Unable to read " + self.fileName + " with FFProbe.")

            for line in p.stderr.readlines():
                self.raw_stderr += str(line)
            self.rawdata = p.communicate()[0]
            self.fileXML = str(search('(<\?xml).*(</ffprobe>)', str(self.rawdata), DOTALL).group(0))
            # print(self.fileXML)
            self.xmlDom = parseString(self.fileXML)


    @property
    def Rawdata(self):
        return self.rawdata


    @property
    def totalRunningTimeRaw(self):
        for stream in self.xmlDom.getElementsByTagName('stream'):
                if stream.getAttribute("codec_type") == "audio":
                    duration = float(stream.getAttribute("duration"))
                    if duration is None or duration == "":
                        raise NoDataException("Could not calculate a duration for " + self.file_name)
                    return duration
        # print self.fileXML
    #     return self.___totalRunningTimeRaw
        pass

    @property
    def totalRunningTimeSMPTE(self):

        # for stream in self.xmlDom.getElementsByTagName('stream'):
        #     if stream.getAttribute("codec_type") == "audio":
        #         seconds = float(stream.getAttribute("duration"))#TODO create SMPTE TRT return
        #         break
        #
        # m, s = divmod(seconds, 60)
        # h, m = divmod(m, 60)
        # return str(int(h)).zfill(2) + ":" + str(int(m)).zfill(2) + ":" + str(int(s)).zfill(2)
        REGEX_DURATION_PATTERN = '(?<=(Duration: ))\d\d:\d\d:\d\d.\d\d(?=,)'
        # print self.raw_stderr
        results = search(REGEX_DURATION_PATTERN, self.raw_stderr).group(0)
        # print
        if results is None or results == "":
            raise NoDataException("Could not calculate the total Running Time in SMPTE for " + self.file_name)
        return results
        pass

    @property
    def totalSeconds(self):
        for stream in self.xmlDom.getElementsByTagName('stream'):
                if stream.getAttribute("codec_type") == "audio":
                    duration = float(stream.getAttribute("duration"))
                    if duration is None or duration == "":
                        raise NoDataException("Could not calculate the number of total seconds for " + self.file_name)
                    return duration
        pass




#!/usr/bin/python
# -*- coding: UTF-8 -*-
from onesheet import XMPMetadata

from abc import ABCMeta
class XMPAudioMetadata(XMPMetadata):
    __metaclass__ = ABCMeta
    def __init__(self):
        print "XMPAudioMetadata start"
        self._getAudioComments = None
        self.___xmpDMAudioSampleRate = None
        self.___xmpDMAudioSampleType = None
        self.___xmpDMAudioChannelType = None
        self.___xmpDMAudioCompressor = None
        self.___xmpDMAudioSpeakerPlacement = None
        self.___xmpDMAbsPeakAudioFilePath = None
        self.___xmpDMRealtivePeakAudioFilePath = None
        self.___xmpDMAudioModDate = None
        self.___xmpDMArtist = None
        self.___xmpDMAlbum = None
        self.___xmpDMTrackNumber = None
        self.___xmpDMGenre = None
        self.___xmpDMCopyright = None
        self.___xmpDMReleaseDate = None
        self.___xmpDMComposer = None
        self.___xmpDMEngineer = None
        self.___xmpDMTempo = None
        self.___xmpDMInstrument = None
        self.___xmpDMIntroTime = None
        self.___xmpDMOutCue = None
        self.___xmpDMRelativeTimestamp = None
        self.___xmpDMLoop = None
        self.___xmpDMNumberofBeats = None
        self.___xmpDMKey = None
        self.___xmpDMStretchMode = None
        self.___xmpDMTimeScaleParams = None
        self.___xmpDMResampleParams = None
        self.___xmpDMBeatSpliceParams = None
        self.___xmpDMTimeSignature = None
        self.___xmpDMScaleType = None

    def getXMPDMAudioSampleRate(self):
        # TODO Make getXMPDMAudioSampleRate method
        return self.___xmpDMAudioSampleRate

    def getXMPDMAudioSampleType(self):
        # TODO Make getXMPDMAudioSampleType method
        return self.___xmpDMAudioSampleType

    def getXMPDMAudioChannelType(self):
        # TODO Make getXMPDMAudioChannelType method
        return self.___xmpDMAudioChannelType

    def getXMPDMAudioCompressor(self):
        # TODO Make getXMPDMAudioCompressor method
        return self.___xmpDMAudioCompressor

    def getXMPDMAudioSpeakerPlacement(self):
        # TODO Make getXMPDMAudioSpeakerPlacement method
        return self.___xmpDMAudioSpeakerPlacement

    def getXMPDMAbsPeakAudioFilePath(self):
        # TODO Make getXMPDMAubsPeakAudio method
        return self.___xmpDMAbsPeakAudioFilePath

    def getXMPDMRealtivePeakAudioFilePath(self):
        # TODO Make getXMPRealtivePeakAudioFilePath method
        return self.___xmpDMRealtivePeakAudioFilePath

    def getXMPDMAudioModDate(self):
        # TODO Make getXMPAudioModData method
        return self.___xmpDMAudioModDate

    def getXMPDMArtist(self):
        # TODO Make getXMPRDMArtist method
        return self.___xmpDMArtist

    def getXMPDMAlbum(self):
        # TODO Make getXMPDMAlbum method
        return self.___xmpDMAlbum

    def getXMPDMTrackNumber(self):
        # TODO Make getXMPDMTrackNumber method
        return self.___xmpDMTrackNumber

    def getXMPDMGenre(self):
        # TODO Make getXMPDMGenre method
        return self.___xmpDMGenre

    def getXMPDMCopyright(self):
        # TODO Make getXMPDMCopyright method
        return self.___xmpDMCopyright

    def getXMPDMReleaseDate(self):
        # TODO Make getXMPDMReleaseDate method
        return self.___xmpDMReleaseDate

    def getXMPDMComposer(self):
        # TODO Make getXMPDMComposer method
        return self.___xmpDMComposer

    def getXMPDMEngineer(self):
        # TODO Make getXMPDMEngineer method
        return self.___xmpDMEngineer

    def getXMPDMTemp(self):
        # TODO Make getXMPMTemp method
        pass

    def getXMPDMInstrument(self):
        # TODO Make getXMPRDMInstrument method
        return self.___xmpDMInstrument

    def getXMPDMIntroTime(self):
        # TODO Make getXMPDMIntroTime method
        return self.___xmpDMIntroTime

    def getXMPDMOutCue(self):
        # TODO Make getXMPRDMOutCue method
        return self.___xmpDMOutCue

    def getXMPDMRelativeTimestamp(self):
        # TODO Make getXMPRealtiveTimestamp method
        return self.___xmpDMRelativeTimestamp

    def getXMPDMLoop(self):
        # TODO Make getXMPDMLoop method
        return self.___xmpDMLoop

    def getXMPDMNumberofBeats(self):
        # TODO Make getXMPNumberOfBeats method
        return self.___xmpDMNumberofBeats

    def getXMPDMKey(self):
        # TODO Make getXMPDMKeys method
        return self.___xmpDMKey

    def getXMPDMStretchMode(self):
        # TODO Make getXMPDStretchMode method
        return self.___xmpDMStretchMode

    def getXMPDMTimeScaleParams(self):
        # TODO Make getXMPDMTimeScaleParams method
        return self.___xmpDMTimeScaleParams

    def getXMPDMResampleParams(self):
        # TODO Make getXMPDResampleParams method
        return self.___xmpDMResampleParams

    def getXMPDMBeatSpliceParams(self):
        # TODO Make getXMPDMBeatSpliceParam method
        return self.___xmpDMBeatSpliceParams

    def getXMPDMTimeSignature(self):
        # TODO Make getXMPDMTimeSignature method
        return self.___xmpDMTimeSignature

    def getXMPDMScaleType(self):
        # TODO Make getXMPDMScaleTypes method
        return self.___xmpDMScaleType




"""
OneSheet
========

OneSheet is a tool for extracting metadata from an Audio, Video, Image, or Document file.

OneSheet is designed for easily creating record generating scripts and scripts for validating files (such as
for quality assurance purposes)


To Install
----------

    $ pip install OneSheet


"""

from setuptools import setup
import subprocess
import sys

try:
    ffprobe = subprocess.check_output(["ffprobe", "-version"])
except OSError:
    sys.stderr.write("Unable to find ffprobe. Make sure that you have ffmpeg install and is in your system path.\n")
    exit(-1)



setup(
    name='OneSheet',
    version='0.1.5.2',
    description='Easily access metadata for image, video, sound, and document file.',
    long_description=__doc__,
    url='https://github.com/henryborchers/OneSheet',
    license='GPL3',
    author='Henry Borchers',
    author_email='henryborchers@yahoo.com',
    packages=['onesheet'],
    classifiers=['Development Status :: 2 - Pre-Alpha',
                 'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
                 'Programming Language :: Python',
                 'Topic :: Multimedia :: Sound/Audio :: Analysis'],
    keywords='metadata extraction'
)
